"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.verifyUrl = exports.verifyHashHex = exports.verifyHashB64 = exports.getPair = exports.getFileExt = exports.stripUrlQuery = exports.domainsToReg = exports.getHostFromUrl = exports.splitList = exports.isHttp = exports.encB64 = exports.decB64 = exports.parseJson = exports.request = exports.linkFile = exports.copyFile = exports.statFile = exports.mkdir = exports.unlinkFile = exports.writeFile = exports.readFileBin = exports.readFile = exports.ver = void 0;
const fs = require("fs");
const dns = require("dns");
const https = require("https");
const node_fetch_1 = require("node-fetch");
// eslint-disable-next-line
exports.ver = require('../package.json').version;
function readFile(file, silent = false) {
    const ret = readFileBin(file, silent);
    return ret && ret.toString();
}
exports.readFile = readFile;
function readFileBin(file, silent = false) {
    try {
        return fs.readFileSync(file);
    }
    catch (err) {
        silent || console.error(err.message);
        return null;
    }
}
exports.readFileBin = readFileBin;
function writeFile(path, data) {
    try {
        fs.writeFileSync(path, data);
        return true;
    }
    catch (err) {
        console.error(err.message);
        return false;
    }
}
exports.writeFile = writeFile;
function unlinkFile(path) {
    try {
        fs.unlinkSync(path);
        return true;
    }
    catch {
        return false;
    }
}
exports.unlinkFile = unlinkFile;
function mkdir(path) {
    try {
        fs.mkdirSync(path, { recursive: true });
        return true;
    }
    catch (err) {
        console.error(err.message);
        return false;
    }
}
exports.mkdir = mkdir;
function statFile(path) {
    try {
        return fs.statSync(path);
    }
    catch (err) {
    }
}
exports.statFile = statFile;
function copyFile(src, dst) {
    try {
        fs.copyFileSync(src, dst);
        return true;
    }
    catch (err) {
        console.error(err.message);
        return false;
    }
}
exports.copyFile = copyFile;
function linkFile(src, dst) {
    try {
        fs.linkSync(src, dst);
        return true;
    }
    catch (err) {
        console.error(err.message);
        return false;
    }
}
exports.linkFile = linkFile;
const mFetchHeaders = {};
let mHttpsAgent;
let mFetchInied = false;
function initFetch() {
    if (mFetchInied) {
        return;
    }
    mFetchHeaders['accept-encoding'] = 'gzip, deflate, br';
    mFetchHeaders['user-agent'] = 'freecdn-cli/' + exports.ver;
    for (const [k, v] of Object.entries(process.env)) {
        const m = k.match(/^FREECDN_HTTP_(.+)/);
        if (m) {
            const name = m[1].toLowerCase().replace(/_/g, '-');
            if (v) {
                mFetchHeaders[name] = v;
            }
            else {
                delete mFetchHeaders[name];
            }
        }
    }
    const RESOVLE = process.env['FREECDN_RESOLVE'];
    if (RESOVLE) {
        const family = +RESOVLE;
        const rrtype = family === 4 ? 'A' : 'AAAA';
        mHttpsAgent = new https.Agent({
            lookup(hostname, opt, callback) {
                dns.resolve(hostname, rrtype, (err, addrs) => {
                    callback(err, addrs[0], family);
                });
            }
        });
    }
    mFetchInied = true;
}
async function request(...args) {
    initFetch();
    const url = args[0];
    let opt = args[1];
    if (!opt) {
        opt = {};
    }
    opt.headers = mFetchHeaders;
    if (mHttpsAgent) {
        console.assert(/^https:/.test(url + ''));
        opt.agent = mHttpsAgent;
    }
    try {
        return await (0, node_fetch_1.default)(url, opt);
    }
    catch (err) {
        console.error(err.message);
    }
}
exports.request = request;
function parseJson(str) {
    try {
        return JSON.parse(str);
    }
    catch {
    }
}
exports.parseJson = parseJson;
function decB64(str) {
    return Buffer.from(str, 'base64');
}
exports.decB64 = decB64;
function encB64(buf) {
    return buf.toString('base64');
}
exports.encB64 = encB64;
function isHttp(url) {
    return /^https?:/.test(url);
}
exports.isHttp = isHttp;
function splitList(str) {
    str = str.trim();
    if (!str) {
        // NOTICE: ''.split(/\s+/) got ['']
        return [];
    }
    return str.split(/\s+/);
}
exports.splitList = splitList;
function getHostFromUrl(url) {
    const m = url.match(/^https?:\/\/([^/]+)/i);
    if (m) {
        return m[1];
    }
    return '';
}
exports.getHostFromUrl = getHostFromUrl;
function domainsToReg(domains) {
    const exps = domains.map(v => v
        .replace(/\./g, '\\.')
        .replace(/\*/g, '.+'));
    try {
        return RegExp('^' + exps.join('|') + '$');
    }
    catch {
    }
}
exports.domainsToReg = domainsToReg;
function stripUrlQuery(url) {
    const pos = url.indexOf('?');
    if (pos === -1) {
        return url;
    }
    return url.substring(0, pos);
}
exports.stripUrlQuery = stripUrlQuery;
function getFileExt(url) {
    const pos = url.lastIndexOf('.');
    if (pos === -1) {
        return '';
    }
    return url.substring(pos + 1);
}
exports.getFileExt = getFileExt;
function getPair(str, char) {
    const pos = str.indexOf(char);
    if (pos === -1) {
        return [str, null];
    }
    return [
        str.substring(0, pos),
        str.substring(pos + char.length)
    ];
}
exports.getPair = getPair;
const REG_SHA256_B64 = /^[A-Za-z0-9+/]{43}=$/;
const REG_SHA256_HEX = /^[A-Fa-f0-9]{64}$/;
function verifyHashB64(value) {
    return REG_SHA256_B64.test(value);
}
exports.verifyHashB64 = verifyHashB64;
function verifyHashHex(value) {
    return REG_SHA256_HEX.test(value);
}
exports.verifyHashHex = verifyHashHex;
function verifyUrl(url) {
    try {
        new URL(url);
        return true;
    }
    catch {
        return false;
    }
}
exports.verifyUrl = verifyUrl;
//# sourceMappingURL=util.js.map